﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

using Pixoneer.NXDL;            // 기본 기능
using Pixoneer.NXDL.NGR;        // 그래픽 관련 기능
using Pixoneer.NXDL.NXMilmap;   // 군사 지도 도시 관련 기능
using Pixoneer.NXDL.NCC;        // 좌표 관련 기능

namespace XDL_MilmapView2
{
    public partial class Form1 : Form
    {
        bool userMeasure = false;
        int countPos = 0;
        XVertex2d posAngle0, posAngle1, posAngle2;    // index 0 : start, index 1 : center, index 2 : end
        public Form1()
        {
            InitializeComponent();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            // XMilmapConfig.xml을 이용하여 군사지도를 위한 NXMilmapView 및 NXMilmapEngine을 초기화한다.
            if (!NXMilmapView.m_MapEngine.InitFromXML("c:\\Pixoneer\\Xdl1.2\\Config\\XMilmapConfig.xml"))
            {
                return;
            }

            nxMilmapView1.AddRenderLayer(ref nxMilmapLayer1);

            // 마우스 휠을 끌고 당기는 것을 확대/축소 요인으로 계산하여 화면을 확대/축소한다.
            // 이 때, 계산된 공간 해상도와 가까운 축척(scale)이 있는 경우 자동적으로 해당 축척으로 바뀌면서 도시된다.
            // 마우스 휠을 끌고 당기는 것으로 화면의 축척을 변경하려면, NXMilmapView.eWheelZoomAction.ByScaleIndex으로 설정
            nxMilmapView1.WheelZoomAction = NXMilmapView.eWheelZoomAction.ByZoomFactor;
            nxMilmapView1.SetGeoToCenter(0, new XVertex2d(127.0, 36.0));

            // MilmapView의 측정 단위 설정
            // 거리 측정 단위
            nxMilmapView1.ToolboxDistUnit = NXMilmapView.eToolboxDistUnit.KiloMeter;
            // 면적 측정 단위
            nxMilmapView1.ToolboxAreaUnit = NXMilmapView.eToolboxAreaUnit.SquareKiloMeter;

            // 사용자 정의 측정을 위한 변수 초기화
            userMeasure = false;
            countPos = 0;
        }

        private void Form1_FormClosed(object sender, FormClosedEventArgs e)
        {
            Xfn.Close();
        }

        private void MeasureLineToolStripMenuItem_Click(object sender, EventArgs e)
        {
            nxMilmapView1.ToolboxMode = NXMilmapView.eToolboxMode.DistanceMeasurer;
        }

        private void MeasurePolyLineToolStripMenuItem_Click(object sender, EventArgs e)
        {
            nxMilmapView1.ToolboxMode = NXMilmapView.eToolboxMode.PathMeasurer;
        }

        private void MeasureAreaToolStripMenuItem_Click(object sender, EventArgs e)
        {
            nxMilmapView1.ToolboxMode = NXMilmapView.eToolboxMode.AreaMeasurer;
        }

        private void MeasureAngleToolStripMenuItem_Click(object sender, EventArgs e)
        {
            nxMilmapView1.ToolboxMode = NXMilmapView.eToolboxMode.AngleMeasurer;
        }

        private void MeasureCircleToolStripMenuItem_Click(object sender, EventArgs e)
        {
            nxMilmapView1.ToolboxMode = NXMilmapView.eToolboxMode.CircleMeasurer;
        }

        private void UserDefinedMeasureToolStripMenuItem_Click(object sender, EventArgs e)
        {
            userMeasure = !userMeasure;
            countPos = 0;
        }

        private bool nxMilmapLayer1_OnWndProc(object sender, Pixoneer.NXDL.NXMilmap.NXMilmapDrawArgs e, ref Message m)
        {
            if (m.Msg == Pixoneer.NXDL.XWndMsg.XWM_LBUTTONDOWN)
            {
                if (userMeasure)
                {
                    double x = Pixoneer.NXDL.XWndMsg.GetLowValue(m.LParam);
                    double y = Pixoneer.NXDL.XWndMsg.GetHighValue(m.LParam);

                    NXMilmapDrawArgs drawArgs = nxMilmapView1.GetDrawArgs();
                    XVertex2d gpPos = drawArgs.ScreenToGeographic(new XVertex2d(x, y));
                    if (countPos == 0)
                    {
                        posAngle0 = gpPos;
                    }
                    else if (countPos == 1)
                    {
                        posAngle1 = gpPos;
                    }
                    else if (countPos == 2)
                    {
                        posAngle2 = gpPos;

                        // 두 선분의 길이와 3 점으로 이루어지는 각도 계산
                        double distance = 0.0;
                        distance += Xcc.CalcGeodeticDistance(XAngle.FromDegree(posAngle0.x), XAngle.FromDegree(posAngle0.y),
                            XAngle.FromDegree(posAngle1.x), XAngle.FromDegree(posAngle1.y));
                        distance += Xcc.CalcGeodeticDistance(XAngle.FromDegree(posAngle1.x), XAngle.FromDegree(posAngle1.y),
                            XAngle.FromDegree(posAngle2.x), XAngle.FromDegree(posAngle2.y));

                        double angle = 0.0;
                        angle = Xcc.CalcGeodeticAngle(XAngle.FromDegree(posAngle1.x), XAngle.FromDegree(posAngle1.y),
                            XAngle.FromDegree(posAngle0.x), XAngle.FromDegree(posAngle0.y),
                            XAngle.FromDegree(posAngle2.x), XAngle.FromDegree(posAngle2.y));

                        System.Diagnostics.Debug.WriteLine("Distance : " + distance.ToString());
                        System.Diagnostics.Debug.WriteLine("Angle    : " + angle.ToString());
                    }

                    countPos++;

                    nxMilmapView1.RefreshScreen();
                }
            }

            return default(bool);
        }

        private bool nxMilmapLayer1_OnRender(object sender, NXMilmapDrawArgs e)
        {
            if (!userMeasure) return false;

            if (countPos > 1)
            {
                e.Graphics.glDisable(XGraphics.GL_DEPTH_TEST);
                e.Graphics.glEnable(XGraphics.GL_BLEND);
                e.Graphics.glBlendFunc(XGraphics.GL_SRC_ALPHA, XGraphics.GL_ONE_MINUS_SRC_ALPHA);

                e.Graphics.glPushMatrix();

                e.Graphics.glColor3f(1.0f, 0.0f, 0.0f);
                e.Graphics.glLineWidth(3);
                e.Graphics.glBegin(XGraphics.GL_LINE_STRIP);

                NXMilmapDrawArgs drawArgs = nxMilmapView1.GetDrawArgs();
                XVertex3d posWorld = drawArgs.GeographicToWorld(posAngle0);
                e.Graphics.glVertex3d(posWorld);

                posWorld = drawArgs.GeographicToWorld(posAngle1);
                e.Graphics.glVertex3d(posWorld);

                if (countPos >= 3)
                {
                    posWorld = drawArgs.GeographicToWorld(posAngle2);
                    e.Graphics.glVertex3d(posWorld);
                }

                e.Graphics.glEnd();
                e.Graphics.glColor3f(1.0f, 1.0f, 1.0f);
                e.Graphics.glPopMatrix();
                e.Graphics.glEnable(XGraphics.GL_DEPTH_TEST);
            }

            return default(bool);
        }
    }
}
